import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import javax.net.ssl.*;

import javax.swing.JOptionPane;

/*
 * Class to connect to a mail server and get headers of
 * messages from the server.  
 */
public class MailConnection {

    // boolean to control debugging output
    private static final boolean DEBUG = true;

    // Well-known port for reading mail
    private static final int PORT = 110;

    // The port if we use a secure connection to pop.gmail.com
    private static final int SECURE_PORT = 995;

    // Remembers if the connection is currently established
    private boolean connected = false;

    // The objects used to read from and write to the mail server
    private BufferedReader input;
    private PrintWriter output;
    private Socket tcpSocket;

    // Create a mail connection to a host for a specific user.
    public MailConnection(String host, String userName, String password) {
        try {
            // gmail and others are all likely to require an ssl 
            // connection to ensure communication is encrypted,
            // so we handle that server differently than fuji.
            if (host.equals("fuji.cs.williams.edu")) {
                // create normal socket
                tcpSocket = new Socket(host, PORT);
            } else {
                // create socket using SSL factory.
                SSLSocketFactory factory = (SSLSocketFactory) SSLSocketFactory.getDefault();
                tcpSocket = (SSLSocket)factory.createSocket(host, SECURE_PORT);
            }

            // Get the stream reader and writer to enable communication
            input =
            new BufferedReader(
                new InputStreamReader(tcpSocket.getInputStream()));
            output =
            new PrintWriter(
                new OutputStreamWriter(tcpSocket.getOutputStream()), true);

            // Add code to exchange messages with the server to login and
            // set "connected" if the login works.

            if (!connected) {
                JOptionPane.showMessageDialog (null, "Login failed");
            }

        } catch (UnknownHostException e) {
            JOptionPane.showMessageDialog(null, "Mail server " + host + " not found.");
        } catch (IOException e) {
            JOptionPane.showMessageDialog(null, "Couldn't connect to mail server.");
        }

        // The connection was not established.  Clean up.
        if (!connected) {
            close();
        }
    }

    // Close both streams and the socket.
    private void close() {
        connected = false;
        if (input != null){
            try {
                input.close();
            } catch (Exception e) {
            }

        }
        if (output != null) {
            try {
                output.close();
            } catch (Exception e) {
            }

        }
        if (tcpSocket != null) {
            try {
                tcpSocket.close();
            } catch (Exception e) {
            }
        }
    }

    // Disconnect from the mail server
    public void disconnect() {
        if (connected) {
            netPrintln("QUIT");
            netReadLine();
            close();
        }
    }

    // Return the number of messages in the mailbox.  Returns 0 if
    // not connected to the mail server.
    public int getNumMsgs() {
        if (connected) {
            // add code to get the number of message available from the
            // server

        }
        return 0; 

    }

    // Ask the mail server for the headers of a message.  Returns an
    // empty string if there is no active connection.
    public String header(int msg) {

        if (connected) {

            // add code to fetch the requested header from the server
            // and modify the next statement to return it

        }
        return "";
    }

    // Returns true if a connection is currently active with the mail server.
    public boolean isConnected() {
        return connected;
    }

    // Read a multi-line response from the mail server.
    // Returns an empty string if there is no connection.
    private String readResponse() {

        if (connected) {
            // add code to read all the lines sent from the
            // server until either a null or a line consisting
            // of a single period is received and return
            // the result of concatenating them together.
        }
        return "";
    }

    // Method used to perform printlns from the network so that
    // a) each line will end with a return followed by a newline as
    // the mailserver expects and b) each line will be logged to
    // System.out for debugging purposes.
    private void netPrintln(String line) { 
        output.println(line + "\r");
        if (DEBUG) System.out.println("Sent     >>" + line);
    }

    // Method used to perform readLines from the network so that
    // each line read will be echoed to System.out for debugging
    // purposes and so that it won't be necessary to handle
    // exceptions every time a line is read.
    private String netReadLine() {
        try {
            String line = input.readLine();
            if (DEBUG) System.out.println("Received >>" + line);
            return line;
        } catch (IOException e) {
            System.out.println("Error: Unable to read from network connection " + e);
            return null;
        }
    }
}