
/*
 * The Message keeps track
 * of the from and subject lines
 * for a single mail message
 */
public class Message {

  private String from;
  private String subject;

  // Construct a Message for the email with the 
  // given headers
  public Message(String headers) {
    from = getLine(headers, "from:");
    subject = getLine(headers, "subject:");
  }
  
  // return the header line starting with the given tag
  private String getLine(String header, String tag) {
    String low = header.toLowerCase();
    int start = low.indexOf("\n" + tag) + 1;
    int end = low.indexOf("\n", start);
    if (end == -1) {
      // last line, so get from start to end of header
      return header.substring(start);
    } else {
      return header.substring(start, end);
    }
  }

  // return the from line from the message's headers
  public String getFrom() {
    return from;
  }

  // return the subject line from the message's headers
  public String getSubject() {
    return subject;
  }

  // return a string containing both the from 
  // and subject lines
  public String toString() {
    return from + "\n" + subject;
  }

}
