
/*
 * The message list manages a list of
 * messages.  It works like all of our
 * collections from this semester by
 * keeping an array and a count of
 * how many entries in the array are being used.
 */
public class MessageList {

  // array of message objects to keep the list
  private Message messages[];

  // number of entries being unsed in the messages array
  private int count = 0;

  // Constructrs a new message with the given maximum size.
  public MessageList(int maxSize) {
    messages = new Message[maxSize];
  }

  // Add a new message to the list 
  public void add(Message newMessage) {
    if (count < messages.length) {
      messages[count] = newMessage;
      count++;
    } else {
      System.out.println("No More Space in Message List!!!");
    }
  }

  public String toString() {
    String result = "";
    for (int i = 0; i < count; i++) {
      result = result + messages[i].toString() + "\n\n";
    }
    return result;
  }

  /*
   * Return a new MessageList containing all of the good messages,
   * which are the ones that do not match any of the filterTerms
   */
  public MessageList getGood(String[] filterTerms) {
    MessageList resultList = new MessageList(messages.length);
    for (int i = 0; i < count; i++) {
      if (!matchExists(filterTerms,messages[i].toString())){
        resultList.add(messages[i]);
      }
    }
    return resultList;
  }

  /*
   * Return a new MessageList containing all of the spam messages,
   * which are the ones that do not match any of the filterTerms
   */
  public MessageList getSpam(String[] filterTerms) {
    MessageList resultList = new MessageList(messages.length);
    for (int i = 0; i < count; i++) {
      if (matchExists(filterTerms,messages[i].toString())){
        resultList.add(messages[i]);
      }
    }
    return resultList;
  }

  /*
   * Return true if the searchString contains any of the filter terms.
   */
  private boolean matchExists(String [] filterTerms, String searchString){
    searchString = searchString.toLowerCase();
    for (int i = 0; i < filterTerms.length; i++){
      if ((!filterTerms[i].equals("")) && searchString.indexOf(filterTerms[i].toLowerCase()) != -1){
        return true;
      }
    }
    return false;
  }
}
