import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import objectdraw.*;

// A simple applet that allows the user to list strings used to filter mail.
// The applet then reads mail from the mailbox and sorts the mail into 
// 2 categories:  good mail and spam.  The user interface provides the
// ability to get mail from a user account, modify the list of
// strings to filter, and view the from and subject lines of good
// mail or of spam.
public class SpamFilter extends Controller implements ActionListener, ItemListener {

  // area used to enter filter terms
  private JTextArea filterArea;   

  // The network connection that allows us to actually read mail
  private MailConnection connection;

  // JComboBox to allow the user to decide whether to show the headers from the
  // good mail or the headers from the spam mail.
  private JComboBox goodOrSpam;

  // Large text area where headers of good mail or spam are shown.
  private JTextArea resultArea;

  // number of messages available on server
  private int messagesAvailable = 0;

  // the next message to read from the server
  private int messageNum = 0;

  // components for the server info
  private JTextField serverField; 
  private JTextField userField; 
  private JPasswordField passwordField;

  // the filter text area
  private JTextArea filters; 

  // components for the buttoms and good/spam choice
  private JButton connectButton, refilterButton; 
  private JComboBox viewMail; 

  // this MessageList keeps a list of all messages
  // from downloaded from the server
  private MessageList allMessages; 

  public void begin() { 
    this.resize(800,600);

    JPanel top = new JPanel(); 
    JPanel left = new JPanel(); 
    JPanel right = new JPanel(); 

    left.setLayout(new GridLayout(5, 1)); 
    left.add(new JLabel("Mail Account Info", JLabel.CENTER)); 

    JPanel tempPanel = new JPanel(); 
    tempPanel.add(new JLabel("Server:")); 
    serverField = new JTextField("fuji.cs.williams.edu", 20); 
    tempPanel.add(serverField); 
    left.add(tempPanel); 

    tempPanel = new JPanel(); 
    tempPanel.add(new JLabel("User:")); 
    userField = new JTextField("", 10); 
    tempPanel.add(userField); 
    left.add(tempPanel); 

    tempPanel = new JPanel(); 
    tempPanel.add(new JLabel("Pass:")); 
    passwordField = new JPasswordField("", 10); 
    tempPanel.add(passwordField); 
    left.add(tempPanel); 

    connectButton = new JButton("Get all mail"); 
    left.add(connectButton); 

    ///////////////////////////////////////////////////////////////////
    // COMMENT OUT THIS LINE TO GET THE PROGRAM TO CONNECT AND LOAD ALL
    // MAIL MESSAGES AT ONCE (AS IN ORIGINAL LAB).
    connectButton.setText("Connect to server");
    //

    right.setLayout(new BorderLayout()); 
    right.add(new JLabel("Filter Terms"), BorderLayout.NORTH); 
    filters = new JTextArea(); 
    right.add(filters, BorderLayout.CENTER); 
    refilterButton = new JButton("Refilter"); 
    right.add(refilterButton, BorderLayout.SOUTH); 

    top.setLayout(new GridLayout(1, 2)); 
    top.add(left); 
    top.add(right); 

    Container content = getContentPane(); 
    content.add(top, BorderLayout.NORTH); 
    JPanel tmp = new JPanel(); 
    viewMail = new JComboBox(); 
    viewMail.addItem("View Good"); 
    viewMail.addItem("View Spam"); 

    tmp.add(viewMail); 
    content.add(tmp, BorderLayout.SOUTH); 

    connectButton.addActionListener(this); 
    refilterButton.addActionListener(this); 
    viewMail.addItemListener(this); 

    JScrollPane pane = new JScrollPane(); 
    resultArea = new JTextArea(); 
    pane.setViewportView(resultArea); 
    content.add(pane, BorderLayout.CENTER); 

    content.validate(); 

  } 

  /*
   * This method performs the filtering, based
   * one whether we want the Good or Spam messages
   * and what terms are in the filter list.
   */
  private void filterMessageList() { 
    if (allMessages != null) { 
      MessageList ml;
      // split will break the string into an array, 
      // with one string for each line.
      String[] filterTerms = filters.getText().split("\n"); 

      if (viewMail.getSelectedItem().equals("View Good")) {
        ml = allMessages.getGood(filterTerms); 
      } else {
        ml = allMessages.getSpam(filterTerms); 
      }

      // show the result in the main text area
      resultArea.setText(ml.toString()); 
    } 
  } 

  // Handles the button clicks to download mail,
  // add and remove filters, and view good mail or spam.
  public void actionPerformed(ActionEvent event) {
    Object which = event.getSource();

    if (which == connectButton) {
      if (connectButton.getText().equals("Get all mail")) {
        connect();
        if (connection.isConnected()) {
          connectButton.setEnabled(false);
          getMessageCount();
          getAllMessages();
          filterMessageList();
          connection.disconnect();
        }
      } else if (connectButton.getText().equals("Connect to server")) {
        connect();
        if (connection.isConnected()){
          connectButton.setText("Get message count");
        }
      } else if (connectButton.getText().equals("Get message count")) {
        getMessageCount();
        messageNum = 0;
        if ( messagesAvailable <= messageNum ) {
          connection.disconnect();
          connectButton.setEnabled(false);
        } else {
          connectButton.setText("Get next message");
        }
      } else if (connectButton.getText().equals("Get next message")) {
        getAMessage(messageNum);
        messageNum++;
        filterMessageList();

        if (messageNum >= messagesAvailable) {
          connectButton.setEnabled(false);
          connection.disconnect();
        }
      }
    } else if (which == refilterButton) {
      filterMessageList();
    } else if (which == goodOrSpam) {
      filterMessageList();
    }

  }

  // Connects to a mail server and gets the headers of all messages.
  // Filters them into good and spam mail and updates the display of whichever
  // is being viewed.  Disconnects from the mail server when done.
  @SuppressWarnings("deprecation")  // This line is needed because of the library we are using...
  private void connect() {
    if (!serverField.getText().equals("")) {
      connection =
      new MailConnection(serverField.getText(), userField.getText(), passwordField.getText());
    }
  }

  private void getMessageCount() {
    messagesAvailable = connection.getNumMsgs();
    allMessages = new MessageList(messagesAvailable);
  }

  private void getAMessage(int num) {
    allMessages.add(new Message(connection.header(num+1)));
  }

  private void getAllMessages() {
    if (connection.isConnected()) {
      for (messageNum = 0; messageNum < messagesAvailable; messageNum++) {
        getAMessage(messageNum);
      }
    }
  }

  /*
  * When the combobox changes, perform the refilter the mail.
   */
  public void itemStateChanged(ItemEvent e) { 
    filterMessageList(); 
  } 

}
